'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (url, _ref) {
  var _ref$authenticated = _ref.authenticated,
      authenticated = _ref$authenticated === undefined ? DEFAULT_OPTIONS.authenticated : _ref$authenticated,
      _ref$body = _ref.body,
      body = _ref$body === undefined ? DEFAULT_OPTIONS.body : _ref$body,
      _ref$csrf = _ref.csrf,
      csrf = _ref$csrf === undefined ? DEFAULT_OPTIONS.csrf : _ref$csrf,
      _ref$method = _ref.method,
      method = _ref$method === undefined ? DEFAULT_OPTIONS.method : _ref$method,
      _ref$tokenName = _ref.tokenName,
      tokenName = _ref$tokenName === undefined ? DEFAULT_OPTIONS.tokenName : _ref$tokenName;


  var config = {
    headers: {
      'Accept': 'application/json',
      'X-Requested-With': 'XMLHttpRequest',
      'Content-Type': 'application/json'
    },
    method: method
  };

  var token = void 0;
  if (authenticated) {
    token = localStorage.getItem(tokenName);
    if (!token) {
      return Promise.reject(new _httpError2.default(401, 'unauthorized', {}));
    }
  }

  if (token) {
    config.headers['Authorization'] = 'Bearer ' + token;
  }

  if (CSRF_METHODS.includes(method)) {

    config.mode = 'same-origin';
    config.credentials = 'include';

    if (csrf) {

      var selector = typeof csrf === 'string' ? csrf : DEFAULT_CSRF_SELECTOR;

      config.headers['X-CSRF-Token'] = csrfToken(selector);
    }
  }

  if (body) {
    config.body = JSON.stringify((0, _utils.decamelizeKeys)(body));
  }

  return fetch(url, config).then(function (response) {
    return response.json().then(function (json) {
      var camelized = (0, _utils.camelizeKeys)(json.data);
      if (response.ok) return camelized;else throw new _httpError2.default(response.status, response.statusText, camelized);
    });
  });
};

var _utils = require('./utils');

var _httpError = require('./http-error');

var _httpError2 = _interopRequireDefault(_httpError);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var CSRF_METHODS = ['PATCH', 'POST', 'PUT'];

var DEFAULT_CSRF_SELECTOR = 'csrf-token';

var DEFAULT_OPTIONS = {
  authenticated: false,
  tokenName: 'token',
  body: null,
  csrf: true,
  method: 'GET'
};

function csrfToken(selector) {
  if (typeof document === 'undefined') return null;

  var token = document.querySelector('meta[name="' + selector + '"]');
  if (token && token instanceof window.HTMLMetaElement) {
    return token.content;
  }
  return null;
}